//#ifdef __OBJC__

#import "MakeupSDK/FrameProvider/FrameProvider.h"
#import <AVFoundation/AVFoundation.h>
#import <CoreGraphics/CoreGraphics.h>
#import <CoreMedia/CoreMedia.h>
#import <CoreVideo/CoreVideo.h>
#import <UIKit/UIKit.h>
#include <mutex>

//#endif

// declare C++ implementation for .m (Obj-C) files
#ifdef __OBJC__
#ifndef __cplusplus
typedef void VsImage;
#endif
#endif

@interface CameraGrabber
    : UIViewController <AVCaptureVideoDataOutputSampleBufferDelegate> {
  AVCaptureSession *_captureSession;
  AVCaptureConnection *_videoConnection;
  const NSString *_preset;

  int _fps;
  int _device;

  NSString *_resolutionId;
  NSNumber *_pixelFormatId;
  AVCaptureVideoOrientation _rotationId;
  BOOL _videoMirrored;

  AVCaptureDeviceInput *_captureInput;
  AVCaptureVideoDataOutput *_captureOutput;
  AVCaptureDevice *_avCaptureDevice;
}

/*!
 @brief	The capture session takes the input from the camera and capture it
 */
@property(nonatomic, retain) AVCaptureSession *captureSession;

@property(nonatomic, retain) AVCaptureConnection *videoConnection;

@property(nonatomic, retain) const NSString *preset;

- (AVCaptureVideoDataOutput *)getCaptureOutput;
- (AVCaptureDevice *)getAVCaptureDevice;
- (id)init:(int)fps:(int)device:(fpr::FrameProvider *)withFrameProvider;
- (void)setResolution:(NSString *)resolutionId;
- (void)setPixelFormat:(NSNumber *)pixelFormatId;
- (void)setRotation:(AVCaptureVideoOrientation)rotation;
- (void)setFlipX:(bool)flipX;

/*!
 @brief	This method initializes the capture session
 */
- (void)initCapture;
- (void)startCapture;
- (void)stopCapture;

@end
