#import "CameraGrabber.h"
#import <mach/mach_time.h>
#import <mutex>
#import <sys/time.h>

@implementation CameraGrabber

@synthesize captureSession = _captureSession;
@synthesize videoConnection = _videoConnection;
@synthesize preset = _preset;

//@synthesize callback = _callback;

int width;
int height;
fpr::FrameProvider *frameprovider;

- (id)init:(int)fps:(int)device:(fpr::FrameProvider *)withFrameProvider {
    self = [super init];
    if (self) {
        frameprovider = withFrameProvider;
        _fps = fps;
        _device = device;
        
        _captureOutput = [[AVCaptureVideoDataOutput alloc] init];
        
        _captureInput = nil;
        NSArray *devices = [AVCaptureDevice devicesWithMediaType:AVMediaTypeVideo];
        for (AVCaptureDevice *device in devices) {
            if (_device == 0) {
                if ([device position] == AVCaptureDevicePositionFront) {
                    _captureInput = [AVCaptureDeviceInput deviceInputWithDevice:device
                                                                          error:nil];
                    break;
                }
            } else {
                if ([device position] == AVCaptureDevicePositionBack) {
                    _captureInput = [AVCaptureDeviceInput deviceInputWithDevice:device
                                                                          error:nil];
                    break;
                }
            }
        }
        
        if (_captureInput == nil) {
            NSLog(@"No camera! Exiting...");
            return self;
        }
        
        _avCaptureDevice = _captureInput.device;
        
        NSError *error2;
        [_avCaptureDevice lockForConfiguration:&error2];
        if (error2 == nil) {
            if (_avCaptureDevice.activeFormat.videoSupportedFrameRateRanges) {
                [_avCaptureDevice setActiveVideoMinFrameDuration:CMTimeMake(1, _fps)];
                [_avCaptureDevice setActiveVideoMaxFrameDuration:CMTimeMake(1, _fps)];
            } else {
                // handle condition
            }
        } else {
            // handle error2
        }
        [_avCaptureDevice unlockForConfiguration];
    }
    
    return self;
}

- (AVCaptureVideoDataOutput *)getCaptureOutput {
    return _captureOutput;
}

- (AVCaptureDevice *)getAVCaptureDevice {
    return _avCaptureDevice;
}

- (void)setResolution:(NSString *)resolutionId {
    _preset = resolutionId;
}

- (void)setPixelFormat:(NSNumber *)pixelFormatId {
    _pixelFormatId = pixelFormatId;
}

- (void)setRotation:(AVCaptureVideoOrientation)rotation {
    _rotationId = rotation;
}

- (void)setFlipX:(bool)flipX {
    _videoMirrored = !flipX;
}

- (void)initCapture {
    _captureOutput.alwaysDiscardsLateVideoFrames = YES;
    
    dispatch_queue_t captureQueue;
    captureQueue =
    dispatch_queue_create("cameraQueue", DISPATCH_QUEUE_CONCURRENT);
    dispatch_set_target_queue(
                              captureQueue, dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_HIGH, 0));
    [_captureOutput setSampleBufferDelegate:self queue:captureQueue];
    
    NSString *key = (NSString *)kCVPixelBufferPixelFormatTypeKey;
    
    NSDictionary *videoSettings =
    [NSDictionary dictionaryWithObject:_pixelFormatId forKey:key];
    [_captureOutput setVideoSettings:videoSettings];
    
    _captureSession = [[AVCaptureSession alloc] init];
    [self.captureSession setSessionPreset:[self.preset copy]];
    [self.captureSession addInput:_captureInput];
    [self.captureSession addOutput:_captureOutput];
    
    AVCaptureConnection *connection =
    [_captureOutput connectionWithMediaType:AVMediaTypeVideo];
    [connection setVideoOrientation:_rotationId];
    connection.videoMirrored = _videoMirrored;
}




#pragma mark -
#pragma mark AVCaptureSession delegate
- (void)captureOutput:(AVCaptureOutput *)captureOutput
didOutputSampleBuffer:(CMSampleBufferRef)sampleBuffer
       fromConnection:(AVCaptureConnection *)connection {
    
    CVImageBufferRef imageBuffer = CMSampleBufferGetImageBuffer(sampleBuffer);
    CVPixelBufferLockBaseAddress(imageBuffer, 0);
    width = (int)CVPixelBufferGetWidth(imageBuffer);
    height = (int)CVPixelBufferGetHeight(imageBuffer);
    
    bool isYuv =(_pixelFormatId == [NSNumber numberWithUnsignedInt:
                                    kCVPixelFormatType_420YpCbCr8BiPlanarFullRange]);
    char *src = isYuv
    ?(char *)CVPixelBufferGetBaseAddressOfPlane(imageBuffer, 0)
    :(char *)CVPixelBufferGetBaseAddress(imageBuffer);
    
    
    CamIL::Image& input = frameprovider->ReuseInputFrame();
    if(input.GetFormat() != CamIL::PredefinedFormat::GetFormat(CamIL::EFormat::BGRA))
        input = CamIL::Image{width, height, CamIL::EFormat::BGRA};
    
    input.GetPlane(0).SetBuffer(src);
    
     frameprovider->AddFrame(input);
    
    CVPixelBufferUnlockBaseAddress(imageBuffer, 0);
}

- (void)startCapture {
    [self initCapture];
    [self.captureSession startRunning];
}

- (void)stopCapture {
    [self.captureSession stopRunning];
    NSLog(@"Stopping camera capture...");
}

#pragma mark -
#pragma mark Memory management

- (void)dealloc {
    [self stopCapture];
}

@end
