//
//  Copyright © 2020 Visage Technologies AB. All rights reserved.
//

#import "ViewController.h"
#import "MakeupSDK/SDKProvider.h"
@interface ViewController ()
@property(strong, nonatomic) IBOutlet UIView *mainView;

@end

@implementation SortedDict

- (id)Get:(id)key {
    return _dict[key];
}

- (void)Set:(id)key object:(id)object {
    _dict[key] = object;
}

- (id)initWithDict:(NSMutableDictionary *)dict andRawJSON:(NSString *)rawJSON {
    _dict = dict;
    
    id sort = ^(NSString *obj1, NSString *obj2) {
        unsigned long location1 = [rawJSON rangeOfString:obj1].location;
        unsigned long location2 = [rawJSON rangeOfString:obj2].location;
        
        return [[NSNumber numberWithUnsignedLong:location1]
                compare:[NSNumber numberWithUnsignedLong:location2]];
    };
    
    _sortedKeys = [[dict allKeys] sortedArrayUsingComparator:sort];
    
    return self;
}

@end

NSMutableDictionary *featureEffects = [[NSMutableDictionary alloc] init];
NSMutableDictionary *featureEffectIds = [[NSMutableDictionary alloc] init];
NSArray *renderOrder = @[
    @"Concealer", @"Foundation", @"Cheekbone blush", @"Contour blush", @"Blush",
    @"Contour", @"Highlighter", @"Eyeshadow1", @"Eyeshadow2", @"Eyeliner",
    @"Mascara", @"Lipstick"
];
int effectIdIndex = 0;
MakeUpEngine *makeUpEngine;
fpr::FrameProvider *frameProvider;
CameraGrabber *camera;

bool cleared = false;
bool showMakeup = true;

SortedDict *looks;
int activeLook = 0;
NSMutableArray *pickerViewImages;

NSMutableDictionary *featureActiveEffect = [[NSMutableDictionary alloc] init];
FeatureTemplate *featureTemplate;
FinishTemplate *finishTemplate;

PickerViewDataProvider *testProvider;
PickerViewDataProviderRegion *regionsLooksProvider;
PickerViewDataProviderLooks *looksProvider;

PickerViewDelegateLooks *looksDelegate;

@implementation EffectObserver

- (void)OnEffectCreated:(MakeUpEngine *)sender:(Effect *)effect {
    [sender DisplayMakeup:effect];
}

- (void)OnEffectUpdated:(MakeUpEngine *)sender:(Effect *)effect {
}

- (void)OnEffectHidden:(MakeUpEngine *)sender:(Effect *)effect {
}

- (void)OnEffectDisplayed:(MakeUpEngine *)sender:(Effect *)effect {
}

- (void)OnEffectRemoved:(MakeUpEngine *)sender:(Effect *)effect {
}

- (void)OnStart:(MakeUpEngine *)sender {
}

- (void)OnClear:(MakeUpEngine *)sender {
    if (!showMakeup) {
        return;
    }
    
    FeatureTemplateRepository *features = [makeUpEngine GetFeatures];
    FinishTemplateRepository *finishes = [makeUpEngine GetFinishes];
    
    int nActive = 0;
    
    for (id feature in renderOrder) {
        int activeEffectIndex = [featureActiveEffect[feature] intValue];
        if (activeEffectIndex > 0) {
            SortedDict *effects = featureEffects[feature];
            
            NSDictionary *effectName = effects.sortedKeys[activeEffectIndex - 1];
            NSDictionary *effect = [effects Get:effectName];
            
            int featureCount = [features GetCount];
            for (int i = 0; i < featureCount; i++) {
                featureTemplate = [features Get:i];
                
                if ([featureTemplate->name isEqualToString:effect[@"feature"]]) {
                    break;
                }
            }
            
            int finishCount = [finishes GetCount];
            for (int i = 0; i < finishCount; i++) {
                finishTemplate = [finishes Get:i];
                
                if ([finishTemplate->name isEqualToString:effect[@"finish"]]) {
                    if ([finishTemplate->name isEqualToString:@"None"]) {
                        finishTemplate = nullptr;
                    }
                    
                    break;
                }
            }
            
            float r = [effect[@"r"] floatValue];
            float g = [effect[@"g"] floatValue];
            float b = [effect[@"b"] floatValue];
            float a = [effect[@"a"] floatValue];
            
            for (ElementTemplate *elementTemplate in featureTemplate->elements) {
                if ([elementTemplate->name isEqualToString:@"Color"] || [elementTemplate->name isEqualToString:@"Smoothing"]) {
                    elementTemplate->color = [[Color alloc] initWithRed:r / 255.0
                                                               andGreen:g / 255.0
                                                                andBlue:b / 255.0
                                                               andAlpha:a / 255.0];
                }
            }
            
            if (finishTemplate != nullptr)
                finishTemplate->transparency = a / 255.0;
            
            [makeUpEngine CreateMakeup:featureTemplate:finishTemplate];
            nActive++;
        }
    }
}

@end

@implementation PickerViewDataProvider

- (id)initWithController:(ViewController *)_controller {
    controller = _controller;
    return self;
}

- (UIImage *)pickerView:(AKPickerView *)pickerView
           imageForItem:(NSInteger)item {
    return pickerViewImages[item];
}

- (NSUInteger)numberOfItemsInPickerView:(AKPickerView *)pickerView {
    return [pickerViewImages count];
}

@end

@implementation PickerViewDataProviderRegion
NSArray *regionsLooks = @[ @"LOOKS" ];

- (NSString *)pickerView:(AKPickerView *)pickerView
            titleForItem:(NSInteger)item {
    return regionsLooks[item];
}

- (NSUInteger)numberOfItemsInPickerView:(AKPickerView *)pickerView {
    return [regionsLooks count];
}
@end

@implementation PickerViewDataProviderLooks {
}
SortedDict *dataSource;

- (id)initWithController:(ViewController *)controller
           andDataSource:(SortedDict *)_dataSource {
    self = [super initWithController:controller];
    
    dataSource = _dataSource;
    
    return self;
}

- (NSString *)pickerView:(AKPickerView *)pickerView
            titleForItem:(NSInteger)item {
    return dataSource.sortedKeys[item];
}

- (UIImage *)pickerView:(AKPickerView *)pickerView
           imageForItem:(NSInteger)item {
    return nil;
}

- (NSUInteger)numberOfItemsInPickerView:(AKPickerView *)pickerView {
    return [dataSource.sortedKeys count];
}

@end

@implementation PickerViewDelegate

- (id)initWithController:(ViewController *)_controller {
    controller = _controller;
    return self;
}

@end

@implementation PickerViewDelegateLooks {
}

- (void)pickerView:(AKPickerView *)pickerView didSelectItem:(NSInteger)item {
    [controller activateLook:[@(item) intValue]];
}

@end

@implementation ViewController

IApplicationLifecycleHandler *handler;

UIView *makeUpView;
StringMessageButton *switchCameraButton;
StringMessageButton *toggleMakeupButton;

int acitveCamera = 0;

AKPickerView *pickerViewRegion;
AKPickerView *pickerViewLooks;

const int pickerViewHeight = 60;

NSArray *eyeFeatures =
@[ @"Eyeshadow1", @"Eyeshadow2", @"Eyeliner", @"Mascara" ];
NSArray *faceFeatures = @[
    @"Contour blush", @"Cheekbone blush", @"Blush", @"Foundation", @"Concealer",
    @"Contour", @"Highlighter"
];
NSArray *lipsFeatures = @[ @"Lipstick" ];

NSDictionary *regions =
@{@"eyes" : eyeFeatures, @"face" : faceFeatures, @"lips" : lipsFeatures};

- (void)viewDidLoad {
    [super viewDidLoad];
    
    [[NSNotificationCenter defaultCenter]
     addObserver:self
     selector:@selector(applicationWillResignActive)
     name:UIApplicationWillResignActiveNotification
     object:nil];
    
    [[NSNotificationCenter defaultCenter]
     addObserver:self
     selector:@selector(applicationDidBecomeActive)
     name:UIApplicationDidBecomeActiveNotification
     object:nil];
    
    [[NSNotificationCenter defaultCenter]
     addObserver:self
     selector:@selector(applicationWillEnterForeground)
     name:UIApplicationWillEnterForegroundNotification
     object:nil];
    
    EffectObserver *effectObserver = [[EffectObserver alloc] init];
    SDKProvider *provider = [SDKProvider GetProvider:@"Data/license.vlc"];
    makeUpEngine = [provider CreateMakeUpEngine];
    frameProvider = [provider GetFrameProvider];
    [makeUpEngine AddObserver:effectObserver];
    handler = [makeUpEngine GetLifecycleHandler];
    
    makeUpView = [makeUpEngine GetView];
    [self.mainView addSubview:makeUpView];
    
    [self startCamera:acitveCamera];
    
    NSString *font = @"WanderyFree-Regular";
    
    const int convenienceButtonSize = 45;
    const int convenienceButtonMargin = 10;
    const int pickerViewHeight = 60;
    const int fontSize = 22;
    
    int screenWidth = self.mainView.frame.size.width;
    int screenHeight = self.mainView.frame.size.height;
    
    switchCameraButton = [StringMessageButton buttonWithType:UIButtonTypeCustom];
    [switchCameraButton addTarget:self
                           action:@selector(switchCameraButtonClicked:)
                 forControlEvents:UIControlEventTouchUpInside];
    UIImage *switchCameraImage =
    [UIImage imageNamed:getImagePath(@"buttons", @"Switch camera")];
    [switchCameraButton setImage:switchCameraImage forState:UIControlStateNormal];
    switchCameraButton.frame =
    CGRectMake(convenienceButtonMargin,
               screenHeight - 2 * pickerViewHeight - convenienceButtonSize -
               convenienceButtonMargin,
               convenienceButtonSize, convenienceButtonSize);
    [self.mainView addSubview:switchCameraButton];
    
    toggleMakeupButton = [StringMessageButton buttonWithType:UIButtonTypeCustom];
    [toggleMakeupButton addTarget:self
                           action:@selector(toggleMakeupButtonClicked:)
                 forControlEvents:UIControlEventTouchUpInside];
    UIImage *toggleMakeupImage =
    [UIImage imageNamed:getImagePath(@"buttons", @"toggle makeup")];
    [toggleMakeupButton setImage:toggleMakeupImage forState:UIControlStateNormal];
    toggleMakeupButton.frame =
    CGRectMake(screenWidth - convenienceButtonSize - convenienceButtonMargin,
               screenHeight - 2 * pickerViewHeight - convenienceButtonSize -
               convenienceButtonMargin,
               convenienceButtonSize, convenienceButtonSize);
    [self.mainView addSubview:toggleMakeupButton];
    
    NSString *rawJSON = [NSString alloc];
    looks = loadJSON(@"looks", &rawJSON);
    
    for (id region in regions) {
        NSDictionary *regionFeatures = [regions objectForKey:region];
        
        for (id feature in regionFeatures) {
            featureEffects[feature] = loadJSON(feature, NULL);
            featureActiveEffect[feature] = 0;
            featureEffectIds[feature] = [[NSMutableDictionary alloc] init];
        }
    }
    
    regionsLooksProvider =
    [[PickerViewDataProviderRegion alloc] initWithController:self];
    looksProvider =
    [[PickerViewDataProviderLooks alloc] initWithController:self
                                              andDataSource:looks];
    looksDelegate = [[PickerViewDelegateLooks alloc] initWithController:self];
    
    pickerViewImages = [[NSMutableArray alloc] init];
    
    CGRect pickerViewTestBounds = CGRectMake(0, screenHeight - pickerViewHeight,
                                             screenWidth, pickerViewHeight);
    pickerViewRegion = [[AKPickerView alloc] initWithFrame:pickerViewTestBounds];
    CGRect pickerViewLooksBounds = CGRectMake(
                                              0, screenHeight - 2 * pickerViewHeight, screenWidth, pickerViewHeight);
    pickerViewLooks = [[AKPickerView alloc] initWithFrame:pickerViewLooksBounds];
    
    pickerViewRegion.dataSource = regionsLooksProvider;
    pickerViewRegion.autoresizingMask =
    UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
    pickerViewRegion.interitemSpacing = 20.0;
    pickerViewRegion.fisheyeFactor = 0.001;
    pickerViewRegion.maskDisabled = false;
    pickerViewRegion.textColor = [UIColor colorWithRed:1.0f
                                                 green:1.0f
                                                  blue:1.0f
                                                 alpha:0.8f];
    pickerViewRegion.highlightedTextColor = [UIColor whiteColor];
    pickerViewRegion.gradient = false;
    pickerViewRegion.backgroundColor = [UIColor colorWithRed:0
                                                       green:0
                                                        blue:0.000001
                                                       alpha:1];
    pickerViewRegion.highlightedFont = [UIFont fontWithName:font size:fontSize];
    pickerViewRegion.font = [UIFont fontWithName:font size:fontSize];
    [self.mainView addSubview:pickerViewRegion];
    
    pickerViewLooks.delegate = looksDelegate;
    pickerViewLooks.dataSource = looksProvider;
    pickerViewLooks.autoresizingMask =
    UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
    pickerViewLooks.interitemSpacing = 20.0;
    pickerViewLooks.fisheyeFactor = 0.001;
    pickerViewLooks.maskDisabled = false;
    pickerViewLooks.textColor = [UIColor colorWithRed:1.0f
                                                green:1.0f
                                                 blue:1.0f
                                                alpha:0.8f];
    pickerViewLooks.highlightedTextColor = [UIColor whiteColor];
    pickerViewLooks.gradient = false;
    pickerViewLooks.backgroundColor = [UIColor colorWithRed:0
                                                      green:0
                                                       blue:0
                                                      alpha:0.75];
    pickerViewLooks.highlightedFont = [UIFont fontWithName:font size:fontSize];
    pickerViewLooks.font = [UIFont fontWithName:font size:fontSize];
    [self.mainView addSubview:pickerViewLooks];
    
    [self activateLook:0];
}

NSString *getImagePath(NSString *folder, NSString *imageName) {
    NSString *bundlePath = [[NSBundle mainBundle] bundlePath];
    NSString *streamingAssetsPath =
    [NSString stringWithFormat:@"%@/Data/Raw/%@/", bundlePath, folder];
    NSString *retValue = [NSString
                          stringWithFormat:@"%@%@.%@", streamingAssetsPath, imageName, @"png"];
    
    return retValue;
}

- (void)viewDidLayoutSubviews {
    [super viewDidLayoutSubviews];
    makeUpView.frame = self.mainView.bounds;
}

- (void)applicationWillResignActive {
    [handler applicationWillResignActive];
    [camera stopCapture];
}

- (void)applicationDidBecomeActive {
    [handler applicationDidBecomeActive];
}

- (void)applicationWillEnterForeground {
    [handler applicationWillEnterForeground];
    [camera startCapture];
}

- (void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [handler applicationWillResignActive];
    [camera stopCapture];
}

- (void)setupCamera:(int)device {
    camera = [[CameraGrabber alloc] init:30:device:frameProvider];
    [camera setRotation:AVCaptureVideoOrientationPortrait];
    [camera setFlipX:false];
    [camera setPixelFormat:[NSNumber
                            numberWithUnsignedInt:kCVPixelFormatType_32BGRA]];
    [camera setResolution:AVCaptureSessionPreset640x480];
    
    [camera startCapture];
}

- (void)startCamera:(int)device {
    NSString *mediaType = AVMediaTypeVideo;
    AVAuthorizationStatus authStatus =
    [AVCaptureDevice authorizationStatusForMediaType:mediaType];
    
    if (authStatus == AVAuthorizationStatusAuthorized) {
        [self setupCamera:device];
    } else if (authStatus == AVAuthorizationStatusDenied) {
        printf("Camera Permission is denied check in settings");
    } else if (authStatus == AVAuthorizationStatusRestricted) {
        // restricted, normally won't happen
    } else if (authStatus == AVAuthorizationStatusNotDetermined) {
        // not determined?!
        [AVCaptureDevice
         requestAccessForMediaType:mediaType
         completionHandler:^(BOOL granted) {
            if (granted) {
                [self setupCamera:device];
            } else {
                NSLog(
                      @"Not granted access to %@, application will not work",
                      mediaType);
            }
        }];
    } else {
        // impossible, unknown authorization status
    }
}

- (IBAction)switchCameraButtonClicked:(StringMessageButton *)sender {
    acitveCamera = 1 - acitveCamera;
    
    [self startCamera:acitveCamera];
}

SortedDict *loadJSON(NSString *feature, NSString **rawJson) {
    NSString *bundle_path = [[NSBundle mainBundle] bundlePath];
    
    NSString *JSON_path = [NSString
                           stringWithFormat:@"%@/Data/Raw/Features/%@.json", bundle_path, feature];
    NSString *JSON_data =
    [[NSString alloc] initWithContentsOfFile:JSON_path
                                    encoding:NSUTF8StringEncoding
                                       error:nil];
    
    NSData *nsdata = [JSON_data dataUsingEncoding:NSUTF8StringEncoding];
    NSError *error;
    NSDictionary *jsonDictionary =
    [NSJSONSerialization JSONObjectWithData:nsdata
                                    options:kNilOptions
                                      error:&error];
    
    return [[SortedDict alloc] initWithDict:(NSMutableDictionary *)jsonDictionary
                                 andRawJSON:JSON_data];
}

- (IBAction)toggleMakeupButtonClicked:(StringMessageButton *)sender {
    showMakeup = !showMakeup;
    [self refreshMakeup];
}

- (void)refreshMakeup {
    [makeUpEngine ClearMakeup];
}

- (void)activateLook:(int)lookIndex {
    activeLook = lookIndex;
    NSDictionary *lookName = looks.sortedKeys[lookIndex];
    NSDictionary *look = [looks Get:lookName];
    
    for (id region in regions) {
        NSDictionary *regionFeatures = [regions objectForKey:region];
        
        for (id feature in regionFeatures) {
            if ([look objectForKey:feature]) {
                SortedDict *effects = featureEffects[feature];
                
                int index = 0;
                for (id effectName in effects.sortedKeys) {
                    if ([effectName isEqualToString:look[feature]]) {
                        break;
                    }
                    index++;
                }
                
                featureActiveEffect[feature] = [NSNumber numberWithInt:index + 1];
            } else {
                featureActiveEffect[feature] = [NSNumber numberWithInt:0];
            }
        }
    }
    
    [self refreshMakeup];
}

UIImage *imageWithImage(UIImage *sourceImage, float height) {
    float oldHeight = sourceImage.size.height;
    float scaleFactor = height / oldHeight;
    
    float newWidth = sourceImage.size.width * scaleFactor;
    float newHeight = oldHeight * scaleFactor;
    
    UIGraphicsBeginImageContext(CGSizeMake(newWidth, newHeight));
    [sourceImage drawInRect:CGRectMake(0, 0, newWidth, newHeight)];
    UIImage *newImage = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    return newImage;
}

- (UIImage *)imageNameToPickerViewImage:(NSString *)imageName {
    UIImage *effectImage =
    [UIImage imageNamed:getImagePath(@"effects", imageName)];
    if (effectImage == nil) {
        effectImage = [UIImage imageNamed:getImagePath(@"effects", @"default.png")];
    }
    
    return imageWithImage(effectImage, pickerViewHeight);
}

@end
