#pragma once

#include "FrameProvider/PlaneDetails.h"
#include "FrameProvider/frameprovider_export.h"
#include <vector>

namespace CamIL {
enum class EFormat {
    RGB = 1,
    RGBA,
    BGR,
    BGRA,
    ARGB,
    ABGR,
    LUMINANCE,
    YUV420p,
    YUV420i
};

/**
 * Defines a format and describes it's structure.
 */
class FRAMEPROVIDER_EXPORT IFormat {
    public:
    IFormat(PlaneDetails* details, int count);

    IFormat() = default;

    ~IFormat() = default;

    /**
     * Checks if two different format objects are describing the same format.
     * @param rhs The format this is compared to.
     */
    bool operator==(IFormat const& rhs) const;

    /**
     * Checks if two different format objects are describing different formats.
     * @param rhs The format this is compared to.
     */
    bool operator!=(IFormat const& rhs) const;

    /**
     * Gets number of planes the image of the specified format has.
     * In case of interlaced formats (e.g. rgba) it's one.
     */
    int GetNumberOfPlanes() const;

    /**
     * Gets details for the specified plane.
     * @param index Specifies index of plane for the result.
     */
    PlaneDetails GetPlaneDetails(int index) const;

    /**
     * Gets width and height of the specified plane for the given width and height of the image.
     * @param index Specifies index of plane for the result.
     * @param width Image width.
     * @param height Image height.
     */
    Dimensions CalculateSizeForPlane(int planeIndex, int width, int height) const;

    protected:
    std::vector<PlaneDetails> planeDetails;
};

namespace PredefinedFormat {
FRAMEPROVIDER_EXPORT const IFormat& GetFormat(EFormat format);
};


} // namespace CamIL
