#pragma once

#include "FrameProvider/PlaneDetails.h"
#include "FrameProvider/frameprovider_export.h"


namespace CamIL {

/**
 * Represents an image retrieved from camera.
 */
class FRAMEPROVIDER_EXPORT Plane {
    public:
    virtual ~Plane() = default;

    /**
     * Returns the buffer size.
     */
    virtual int GetSize() const = 0;

    /**
     * Returns the image width.
     */
    virtual int GetWidth() const = 0;

    /**
     * Returns the image height.
     */
    virtual int GetHeight() const = 0;

    /**
     * Returns bytes per pixel of image.
     */
    virtual float GetBytesPerPixel() const = 0;

    /**
     * Reads byte on the given index.
     * @param index Buffer index to write to. Must be a valid index smaller than the plane buffer size.
     */
    virtual char GetByte(int index) = 0;

    /**
     * Writes byte to a given index.
     * @param index Buffer index to write to. Must be a valid index smaller than the plane buffer size.
     * @param byte Value to write.
     */
    virtual void SetByte(int index, char byte) = 0;


    /**
     * Gets the detaiols of the plane.
     */
    virtual PlaneDetails GetDetails() = 0;

    /**
     * Returns a pointer to the buffer with the image data.
     */
    virtual void* GetBuffer() const = 0;

    /**
     * Copies data into buffer.
     * @param data pointer to the data to copy. It must be same size of this plane's buffer.
     */
    virtual void SetBuffer(void* data) = 0;
};
} // namespace CamIL
