#pragma once
#include "FrameProvider/frameprovider_export.h"
#include <string>

namespace CamIL {
struct FRAMEPROVIDER_EXPORT Dimensions {
    public:
    Dimensions(int width, int height) : width(width), height(height){};

    Dimensions() = default;

    int GetWidth() const { return width; }
    int GetHeight() const { return height; }

    private:
    int width = 0;
    int height = 0;
};

/**
 * Describes details about the plane.
 */
class FRAMEPROVIDER_EXPORT PlaneDetails {
    public:
    /**
     * Creates new plane details object.
     * @param samplingRate The plane dimensions.
     * @param bytesPerPixel Bytes per pixel of the plane.
     * @param name The plane name.
     */
    PlaneDetails(Dimensions samplingRate, int bytesPerPixel, const char* name)
    : samplingRate(samplingRate), bytesPerPixel(bytesPerPixel),
      name(std::string(name)){};

    PlaneDetails() = default;

    /**
     * Checks if two plane details are equal.
     * @param rhs The plane details this is compared to.
     */
    bool operator==(const PlaneDetails& rhs) {
        if(this->GetWidthSampling() != rhs.GetWidthSampling()) {
            return false;
        }

        if(this->GetHeightSampling() != rhs.GetHeightSampling()) {
            return false;
        }

        if(this->GetBytesPerPixel() != rhs.GetBytesPerPixel()) {
            return false;
        }

        if(std::string(this->name) != std::string(rhs.name)) {
            return false;
        }

        return true;
    }

    /**
     * Checks if two plane details are different.
     * @param rhs The plane details this is compared to.
     */
    bool operator!=(const PlaneDetails& rhs) { return !(*this == rhs); }

    /**
     * Gets plane width sampling.
     */
    int GetWidthSampling() const { return samplingRate.GetWidth(); }
    /**
     * Gets plane height sampling.
     */
    int GetHeightSampling() const { return samplingRate.GetHeight(); }
    /**
     * Gets plane bytes per pixel.
     */
    int GetBytesPerPixel() const { return bytesPerPixel; }
    /**
     * Gets plane name.
     */
    const char* GetName() const { return name.c_str(); }

    private:
    Dimensions samplingRate;
    int bytesPerPixel = 0;
    std::string name;
};
} // namespace CamIL
