#pragma once

#include "FrameProvider/Format.h"
#include "FrameProvider/Plane.h"
#include "FrameProvider/PlaneDetails.h"
#include "FrameProvider/frameprovider_export.h"


namespace CamIL {

/**
 * Represents an image retrieved from camera.
 */
class PlaneInternal : public Plane {
    public:
    /**
     * Constructor with parameters. Creates a deep copy of the source buffer.
     * @param width Width of the image.
     * @param height Height of the image.
     * @buffer source buffer with image data.
     * @param bufferSize size of the buffer.
     */
    PlaneInternal(int width, int height, void* buffer, const PlaneDetails& details);

    /**
     * Constructor with parameters. Creates a empty buffer.
     * @param width Width of the image.
     * @param height Height of the image.
     * @param bufferSize size of the buffer.
     */
    PlaneInternal(int width, int height, const PlaneDetails& details);

    /**
     * Constructor with parameters. Creates a empty buffer.
     * @param width Width of the image.
     * @param height Height of the image.
     * @param bufferSize size of the buffer.
     */
    PlaneInternal(int width, int height, PlaneDetails&& details);

    /**
     * Default constructor.
     */
    PlaneInternal() = default;

    virtual ~PlaneInternal() override = default;

    /**
     * Returns the buffer size.
     */
    int GetSize() const override;

    /**
     * Returns the image width.
     */
    int GetWidth() const override;

    /**
     * Returns the image height.
     */
    int GetHeight() const override;

    float GetBytesPerPixel() const override;

    char GetByte(int index) override;

    void SetByte(int index, char byte) override;

    PlaneDetails GetDetails() override;

    /**
     * Returns a pointer to the buffer with the image data.
     */
    void* GetBuffer() const override;

    void SetBuffer(void* data) override;

    private:
    int width = 0;
    int height = 0;
    PlaneDetails details;

    // doesn't own
    void* buffer;
};
} // namespace CamIL