#import <UIKit/UIKit.h>
#import "FeatureTemplate.h"

@class MakeUpEngine;

/**
 * Protocol.
 * Exposes events triggered by MakeupEngine.
 * The API client should implement this protocol in order to receve notifications from the engine.
 */
@protocol IMakeUpObserver

/**
 * Triggered when the application is started.
 * @param sender an instance of makeup engine
 */
- (void)OnStart:(MakeUpEngine*)sender;

/**
 * Triggered when queried makeup feature is created.
 * @param sender an instance of makeup engine
 * @param effect created effect
 */
- (void)OnEffectCreated:(MakeUpEngine*)sender :(Effect*)effect;

/**
 * Triggered when makeup feature is updated.
 * @param sender an instance of makeup engine
 * @param effect updated effect
 */
- (void)OnEffectUpdated:(MakeUpEngine*)sender :(Effect*)effect;

/**
 * Triggered when makeup feature is removed.
 * @param sender an instance of makeup engine
 * @param effect removed effect
 */
- (void)OnEffectRemoved:(MakeUpEngine*)sender :(Effect*)effect;

/**
 * Triggered when makeup feature is hidden.
 * @param sender an instance of makeup engine
 * @param effect hidden effect
 */
- (void)OnEffectHidden:(MakeUpEngine*)sender :(Effect*)effect;

/**
 * Triggered when makeup feature is displayed.
 * @param sender an instance of makeup engine
 * @param effect displayed effect
 */
- (void)OnEffectDisplayed:(MakeUpEngine*)sender :(Effect*)effect;

/**
 * Triggered when makeup features are cleared.
 * @param sender an instance of makeup engine
 */
- (void)OnClear:(MakeUpEngine*)sender;

@end
